unit Unit1;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Windows;

type

  { TBorderlessForm }

  TBorderlessForm = class(TForm)
    Button1: TButton;
    CustomTitleBar: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure CustomTitleBarDblClick(Sender: TObject);
    procedure CustomTitleBarMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
  private
    function IsBorderless: Boolean;
    function GetBorderSpace: Integer;
    procedure WM_NCHitTest(var Msg: TWMNCHitTest); message WM_NCHITTEST;
    procedure WM_NCCalcSize(var Msg: TWMNCCalcSize); message WM_NCCALCSIZE;

  protected
    procedure Paint; override;
//    procedure Resize; override;
  public

  end;

var
  BorderlessForm: TBorderlessForm;

implementation

{$R *.lfm}

{ TBorderlessForm }

procedure TBorderlessForm.CustomTitleBarDblClick(Sender: TObject);
begin
  if WindowState = wsNormal then
    WindowState := wsMaximized
  else
    WindowState := wsNormal;
end;

procedure TBorderlessForm.Button1Click(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TBorderlessForm.CustomTitleBarMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  ReleaseCapture();
  SendMessage(Self.Handle, WM_SYSCOMMAND, 61458, 0) ;
end;

procedure TBorderlessForm.FormCreate(Sender: TObject);
begin
end;

function TBorderlessForm.IsBorderless: Boolean;
begin
  Result := BorderStyle in [bsNone, bsToolWindow, bsSizeToolWin];
end;

function TBorderlessForm.GetBorderSpace: Integer;
begin
  case BorderStyle of
     bsSingle:
       Result :=  GetSystemMetrics(SM_CYFIXEDFRAME);
     bsDialog, bsToolWindow:
       Result :=  GetSystemMetrics(SM_CYDLGFRAME);
     bsSizeable, bsSizeToolWin:
       Result :=  GetSystemMetrics(SM_CYSIZEFRAME) +
                  GetSystemMetrics(SM_CXPADDEDBORDER);
     else
       Result := 0;
   end;
end;

procedure TBorderlessForm.WM_NCCalcSize(var Msg: TWMNCCalcSize);
var
  CaptionBarHeight: Integer;
begin
  inherited;
  if BorderStyle = bsNone then exit;
  CaptionBarHeight := GetSystemMetrics(SM_CYCAPTION);
  if WindowState = wsNormal then
    Inc(CaptionBarHeight, GetBorderSpace);
  Dec(Msg.CalcSize_Params^.rgrc[0].Top, CaptionBarHeight);
end;

procedure TBorderlessForm.WM_NCHitTest(var Msg: TWMNCHitTest);
var
  ResizeSpace: Integer;
begin
  inherited;
  ResizeSpace := GetBorderSpace;

  if
    (WindowState = wsNormal) and
    (BorderStyle in [bsSizeable, bsSizeToolWin]) and
    (Msg.YPos - BoundsRect.Top <= ResizeSpace)
  then
    begin
      if Msg.XPos - BoundsRect.Left <= 2 * ResizeSpace then
        Msg.Result := HTTOPLEFT
      else if BoundsRect.Right - Msg.XPos <= 2 * ResizeSpace then
        Msg.Result := HTTOPRIGHT
      else
        Msg.Result := HTTOP;
    end;
end;

procedure TBorderlessForm.Paint;
begin
  inherited;
  if (WindowState = wsNormal) and (not IsBorderless) then
    begin
      Canvas.Pen.Color := clRed;
      Canvas.MoveTo(0, 0);
      Canvas.LineTo(Width, 0);
    end;
end;

{procedure TBorderlessForm.Resize;
begin
//  inherited;
  //if (WindowState = wsNormal) and (not IsBorderless) then
  //  Padding.Top := 1
  //else
  //  Padding.Top := 0;
end;
}
end.

